#
# Copyright (c) 2020-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#


import os
import sys
import psutil
from netq_lib.orm.redisdb.models import AgentStats
from netq_lib.common.enums import ServiceStatus
from netq_lib.common.agent_stats_monitor import AgentStatsMon
from netq_agent.parsers.common.render import BaseRender


class AgentStatsRender(BaseRender):
    def __init__(self):
        super(AgentStatsRender, self).__init__()
        self.model = AgentStats

    def parse(self, cmd_output, svc_failed=False, **kwargs):
        agent_stats_obj = AgentStats()
        agent_stats_mon_obj = AgentStatsMon.get_instance()

        agent_stats_obj.agent_util = {'agent_pid': agent_stats_mon_obj.agent_util_stats.get('agent_pid', os.getpid()),
                                      'max_cpu_util': agent_stats_mon_obj.agent_util_stats.get('max_cpu_util', 0.0),
                                      'min_cpu_util': agent_stats_mon_obj.agent_util_stats.get('min_cpu_util', 0.0),
                                      'avg_cpu_util': agent_stats_mon_obj.agent_util_stats.get('sum_cpu_util', 0.0) / agent_stats_mon_obj.agent_util_stats.get('count', 1),
                                      'max_mem_util': agent_stats_mon_obj.agent_util_stats.get('max_mem_util', 0.0),
                                      'min_mem_util': agent_stats_mon_obj.agent_util_stats.get('min_mem_util', 0.0),
                                      'avg_mem_util': agent_stats_mon_obj.agent_util_stats.get('sum_mem_util', 0.0) / agent_stats_mon_obj.agent_util_stats.get('count', 1)}

        agent_stats_obj.message_count = agent_stats_mon_obj.message_count
        agent_stats_obj.msg_type_count = agent_stats_mon_obj.msg_type_count
        agent_stats_obj.msgtype_fullstate_count = agent_stats_mon_obj.msgtype_fullstate_count
        agent_stats_obj.pq_full_count = agent_stats_mon_obj.pq_full_count
        agent_stats_obj.agent_pause_count = agent_stats_mon_obj.agent_pause_count

        agent_stats_mon_obj.reset_agent_stats_counters()

        agent_stats_obj.save()
        return ServiceStatus.OK


class AgentUtilStatsRender(BaseRender):
    def __init__(self): #pylint: disable=useless-super-delegation
        super(AgentUtilStatsRender, self).__init__()

    def parse(self, cmd_output, svc_failed=False, **kwargs):
        agent_util_mon = AgentStatsMon.get_instance()
        agent_process = psutil.Process(os.getpid())

        agent_cpu_usage = agent_process.cpu_percent(interval=0.1)
        total_mem = psutil.virtual_memory().total
        agent_mem_usage = float((agent_process.memory_info().vms * 100)) / total_mem

        agent_util_mon.agent_util_stats['agent_pid'] = os.getpid()
        agent_util_mon.agent_util_stats['count'] = agent_util_mon.agent_util_stats.get('count', 0) + 1


        if agent_cpu_usage >= agent_util_mon.agent_util_stats.get('max_cpu_util', 0.0):
            agent_util_mon.agent_util_stats['max_cpu_util'] = agent_cpu_usage

        if agent_cpu_usage <= agent_util_mon.agent_util_stats.get('min_cpu_util', sys.maxsize):
            agent_util_mon.agent_util_stats['min_cpu_util'] = agent_cpu_usage


        agent_util_mon.agent_util_stats['sum_cpu_util'] = agent_util_mon.agent_util_stats.get('sum_cpu_util', 0.0) +\
                                                          agent_cpu_usage

        if agent_mem_usage >= agent_util_mon.agent_util_stats.get('max_mem_util', 0.0):
            agent_util_mon.agent_util_stats['max_mem_util'] = agent_mem_usage

        if agent_mem_usage <= agent_util_mon.agent_util_stats.get('min_mem_util', sys.maxsize):
            agent_util_mon.agent_util_stats['min_mem_util'] = agent_mem_usage

        agent_util_mon.agent_util_stats['sum_mem_util'] = agent_util_mon.agent_util_stats.get('sum_mem_util', 0.0) +\
                                                          agent_mem_usage

        return ServiceStatus.OK
