#
# Copyright (c) 2019-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#


import os

from netq_lib.orm import HOSTNAME
from netq_lib.orm.redisdb.models import ClSupport
from netq_lib.common.enums import ServiceStatus
from netq_lib.common.utils import save_fullstate
from netq_agent.parsers.common.render import BaseRender


class ClSupportRender(BaseRender):
    def __init__(self): #pylint: disable=useless-super-delegation
        super(ClSupportRender, self).__init__()

    def build_fullstate_clobj(self, new_file, full_state_arr):
        new_cl_support_objs = ClSupport()
        new_cl_support_objs.file_abs_name = new_file
        new_cl_support_objs.create_time = os.path.getctime(new_file)
        full_state_arr.append(new_cl_support_objs)

    def parse(self, cmd_output, svc_failed=False, **kwargs):
        filepath = self._get_filepath()
        file_to_monitor_start_name = self._get_file_start_name()
        file_to_monitor_end_name = self._get_file_end_name()

        fullstate = kwargs.get('fullstate')

        # grab current list of clSupport objects
        m_clsupport = ClSupport.memmodel()
        old_cl_support_objs = []
        new_cl_support_objs = []
        if m_clsupport:
            old_cl_support_objs = [old_obj.file_abs_name for old_obj in m_clsupport.query.filter(hostname=HOSTNAME)]

        full_state_arr = []
        if os.path.isdir(filepath):
            for new_entry in [name for name in os.listdir(filepath) if not os.path.isdir(os.path.join(filepath, name))]:
                new_file = filepath + "/" + new_entry
                if not (new_entry.startswith(file_to_monitor_start_name) and new_entry.endswith(file_to_monitor_end_name)):
                    continue
                if fullstate:
                    self.build_fullstate_clobj(new_file, full_state_arr)
                elif new_file not in old_cl_support_objs:
                    new_cl_support_objs = ClSupport()
                    new_cl_support_objs.file_abs_name = new_file
                    new_cl_support_objs.create_time = os.path.getctime(new_file)
                    new_cl_support_objs.save()

        if fullstate:
            save_fullstate(full_state_arr, ClSupport().__class__.__name__)

        return ServiceStatus.OK
