#
# Copyright (c) 2020-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#


import os
from netq_lib.orm import HOSTNAME
from netq_lib.orm.redisdb.models import ConfigMon
from netq_lib.common.enums import ServiceStatus
from netq_lib.common.utils import save_fullstate
from netq_agent.parsers.common.render import BaseRender


class ConfigMonRender(BaseRender):
    def __init__(self):
        super(ConfigMonRender, self).__init__()
        self.model = ConfigMon
        self._logger = None

    def get_config_dict(self):
        raise NotImplementedError

    def build_platform_specific_objs(self, filename, filepath, fullstate, mem_modify_time, current_modify_time):
        raise NotImplementedError

    def has_special_handling(self, filename):
        return False

    def get_last_modify_time(self, filename, filepath):
        raise NotImplementedError

    def get_mem_obj_last_modify_time(self, filename, old_configmon_objs):
        mem_modify_time = 0
        for old_entry in old_configmon_objs:
            if old_entry.file_name == filename:
                mem_modify_time = int(old_entry.last_modify_time)
                break
        return mem_modify_time

    def parse(self, cmd_output, svc_failed=False, **kwargs):
        self._logger = kwargs.get('logger')
        try:
            conf_dict = self.get_config_dict()
        except ValueError:
            m_configMon = ConfigMon.memmodel()
            for entry in m_configMon.query.filter(hostname=HOSTNAME):
                old_config_mon_ent = ConfigMon()
                old_config_mon_ent.copy(entry)
                old_config_mon_ent.delete()
            return ServiceStatus.FAIL

        fullstate = kwargs.get('fullstate')

        # grab current list of configMon objects
        m_configMon = ConfigMon.memmodel()
        old_configmon_objs = []
        full_state_arr = []
        deleteFileList = []
        if m_configMon:
            old_configmon_objs = (
                list(m_configMon.query.filter(hostname=HOSTNAME))
            )
        if conf_dict.get('config-diff'):
            for filename, filepath in list(conf_dict.get('config-diff').items()):
                if os.path.isfile(filepath):
                    mem_modify_time = self.get_mem_obj_last_modify_time(filename, old_configmon_objs)
                    current_modify_time = os.path.getmtime(filepath)

                    if int(current_modify_time) > mem_modify_time or fullstate:
                        new_config_mon_obj = ConfigMon()
                        new_config_mon_obj.file_name = filename
                        new_config_mon_obj.last_modify_time = current_modify_time

                        with open(filepath, 'rb') as myfile:
                            file_content = myfile.read().decode()

                        new_config_mon_obj.file_content = file_content
                        if not fullstate:
                            new_config_mon_obj.save()
                        else:
                            full_state_arr.append(new_config_mon_obj)
                elif self.has_special_handling(filename):
                    mem_modify_time = self.get_mem_obj_last_modify_time(filename, old_configmon_objs)
                    current_modify_time = self.get_last_modify_time(filename, filepath)
                    if not current_modify_time:
                        continue
                    new_config_mon_obj = self.build_platform_specific_objs(filename, filepath, fullstate, mem_modify_time, current_modify_time)
                    if not new_config_mon_obj:
                        continue
                    if not fullstate:
                        new_config_mon_obj.save()
                    else:
                        full_state_arr.append(new_config_mon_obj)
                else:
                    deleteFileList.append(filename)
        for to_be_deleted_file in deleteFileList:
            for old_obj in old_configmon_objs:
                if old_obj.file_name == to_be_deleted_file:
                    old_configmon_ent = ConfigMon()
                    old_configmon_ent.copy(old_obj)
                    old_configmon_ent.delete()
                    break


        if fullstate:
            save_fullstate(full_state_arr, ConfigMon().__class__.__name__)

        return ServiceStatus.OK
