#
# Copyright (c) 2020-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#

from netq_lib.orm import HOSTNAME
from netq_lib.orm.redisdb.models import PackageInfo
from netq_lib.common.enums import ServiceStatus
from netq_lib.common.utils import save_fullstate
from netq_agent.parsers.common.render import BaseRender


class DpkgRender(BaseRender):
    def __init__(self):
        super(DpkgRender, self).__init__()
        self.model = PackageInfo
        self._logger = None

    def parse(self, cmd_output, svc_failed=False, **kwargs):
        """Build dpkg info
        """
        fullstate = kwargs.get('fullstate')
        full_state_arr = []
        self._logger = kwargs.get('logger')

        m_dpkg = PackageInfo.memmodel()
        old_dpkg_objs = list(m_dpkg.query.filter(hostname=HOSTNAME))
        old_pkg_names = {}
        for elem in old_dpkg_objs:
            old_pkg_names[elem.package_name] = elem

        cl_version = self.get_os_version()
        if cmd_output:
            command_result = cmd_output.split("\n")
            if len(old_dpkg_objs) == 0: #pylint: disable=len-as-condition
                for individual_element in command_result:
                    (flag, package_info_obj) = self.get_dpkg_obj(individual_element, cl_version)
                    if flag:
                        full_state_arr.append(package_info_obj)
                        package_info_obj.save()
            else:
                for individual_element in command_result:
                    package_info_nm = self.get_package_name(individual_element)
                    if package_info_nm in old_pkg_names:
                        # old_obj_flag = False
                        old_pkg_names.pop(package_info_nm)
                    (flag, package_info_obj) = self.get_dpkg_obj(individual_element, cl_version)
                    if flag:
                        full_state_arr.append(package_info_obj)
                        package_info_obj.save()

                for completeElem in old_pkg_names:
                    elem = old_pkg_names[completeElem]
                    package_info_obj = PackageInfo()
                    package_info_obj.package_name = elem.package_name
                    package_info_obj.version = elem.version
                    package_info_obj.status = elem.status
                    package_info_obj.deleted = True
                    package_info_obj.cl_version = cl_version
                    package_info_obj.save()
                    full_state_arr.append(package_info_obj)

        if fullstate:
            save_fullstate(full_state_arr, PackageInfo.__name__)

        return ServiceStatus.OK

    def get_package_name(self, individual_element):
        package_info_obj_elements = individual_element.split(",")
        if len(package_info_obj_elements) == 3:
            return package_info_obj_elements[0]
        return "NA"

    def get_dpkg_obj(self, individual_element, cl_version):
        package_info_obj = PackageInfo()
        package_info_obj_elements = individual_element.split(",")
        if len(package_info_obj_elements) == 3:
            package_info_obj.package_name = package_info_obj_elements[0]
            package_info_obj.version = package_info_obj_elements[1]
            package_info_obj.status = package_info_obj_elements[2]
        else:
            return (False, package_info_obj)
        package_info_obj.cl_version = cl_version
        return (True, package_info_obj)
