#
# Copyright (c) 2020-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#


import os
import subprocess
from netq_lib.orm import HOSTNAME
from netq_lib.orm.redisdb.models import NvueMon
from netq_lib.common.enums import ServiceStatus
from netq_lib.common.utils import save_fullstate, is_active_systemd_svc
from netq_agent.parsers.common.render import BaseRender


class NvueMonRender(BaseRender):
    def __init__(self):
        super(NvueMonRender, self).__init__()
        self.model = NvueMon
        self._logger = None

    def get_config_dict(self):
        raise NotImplementedError

    def build_platform_specific_objs(self, filename, filepath, fullstate, mem_modify_time, current_modify_time):
        raise NotImplementedError

    def has_special_handling(self, filename):
        return False

    def get_last_modify_time(self, filename, filepath):
        raise NotImplementedError

    def get_mem_obj_last_modify_time(self, filename, old_configmon_objs):
        mem_modify_time = 0
        for old_entry in old_configmon_objs:
            if old_entry.file_name == filename:
                mem_modify_time = int(old_entry.last_modify_time)
                break
        return mem_modify_time

    def get_mem_obj_last_rev_id(self, filename, old_nvuemon_objs):
        mem_rev_id = ""
        for old_entry in old_nvuemon_objs:
            if old_entry.file_name == filename:
                mem_rev_id = old_entry.rev_id
                break
        return mem_rev_id

    def parse(self, cmd_output, svc_failed=False, **kwargs):
        os.environ['HOME'] = "/root"
        self._logger = kwargs.get('logger')
        try:
            if os.path.exists('/bin/systemctl'):
                is_active = is_active_systemd_svc('nvued')

                if not is_active:
                    return ServiceStatus.OK

        except Exception as e:
            self._logger.debug("nvued is not active")
            return ServiceStatus.OK

        try:
            conf_dict = self.get_config_dict()
        except ValueError:
            m_nvueMon = NvueMon.memmodel()
            for entry in m_nvueMon.query.filter(hostname=HOSTNAME):
                old_nvue_mon_ent = NvueMon()
                old_nvue_mon_ent.copy(entry)
                old_nvue_mon_ent.delete()
            return ServiceStatus.FAIL

        fullstate = kwargs.get('fullstate')

        # grab current list of configMon objects
        m_nvueMon = NvueMon.memmodel()
        old_nvuemon_objs = []
        full_state_arr = []
        deleteFileList = []
        if m_nvueMon:
            old_nvuemon_objs = (
                list(m_nvueMon.query.filter(hostname=HOSTNAME))
            )
        if conf_dict.get('nvue-diff'):
            for filename, filepath in list(conf_dict.get('nvue-diff').items()):
                mem_rev_id = self.get_mem_obj_last_rev_id(filename, old_nvuemon_objs)
                CMD = "/usr/bin/nv config history | grep rev_id | sed -n '1,1p'"
                #output = subprocess_wrapper(CMD)
                try:
                    output = subprocess.check_output(CMD, timeout=10, shell=True)
                except Exception as e:
                    self._logger.debug(str(e)) #pylint: disable=no-member
                    return ServiceStatus.FAIL

                rev_id = output.decode()
                curr_rev_id = ""
                if rev_id:
                    curr_rev_id = rev_id.split(":")[1].strip()

                if curr_rev_id != mem_rev_id or fullstate:
                    if curr_rev_id == "":
                        return ServiceStatus.OK
                    new_nvue_mon_obj = NvueMon()
                    new_nvue_mon_obj.file_name = filename
                    DATE_CMD = "/usr/bin/nv config history | grep date: | sed -n '1,1p'"
                    #current_modify_time = subprocess_wrapper(DATE_CMD).decode().split(":")[1].strip()
                    try:
                        current_modify_time = subprocess.check_output(DATE_CMD, timeout=10, shell=True)
                        new_nvue_mon_obj.last_modify_time = current_modify_time.decode().split("'")[1]
                    except Exception as e:
                        self._logger.debug(str(e))  # pylint: disable=no-member
                        return ServiceStatus.FAIL
                    new_nvue_mon_obj.rev_id = curr_rev_id
                    NV_CMD = "/usr/bin/nv config show"
                    #file_content = NV_CMD
                    #file_content = subprocess_wrapper(NV_CMD.split())
                    #file_content = subprocess_wrapper(NV_CMD).decode()
                    try:
                        file_content = subprocess.check_output(NV_CMD, shell=True)
                        new_nvue_mon_obj.file_content = file_content.decode()
                    except Exception as e:
                        self._logger.debug(str(e)) #pylint: disable=no-member
                        return ServiceStatus.FAIL
                    if not fullstate:
                        new_nvue_mon_obj.save()
                    else:
                        full_state_arr.append(new_nvue_mon_obj)

        if fullstate:
            save_fullstate(full_state_arr, NvueMon().__class__.__name__)

        return ServiceStatus.OK
