#
# Copyright (c) 2020-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#


import os
import yaml
import psutil
from netq_lib.orm.redisdb.models import ResourceUtil
from netq_lib.common.enums import ServiceStatus
from netq_lib.common.utils import save_fullstate
from netq_agent.parsers.common.render import BaseRender
from netq_lib.common.enums import DefaultPaths

class ResourceUtilRender(BaseRender):
    _DISK_IGNORE_CACHE = dict()
    def __init__(self):
        super(ResourceUtilRender, self).__init__()
        self.model = ResourceUtil

    def parse(self, cmd_output, svc_failed=False, **kwargs):
        fullstate = kwargs.get('fullstate')
        period = kwargs.get('period')
        cpu_times = psutil.cpu_times_percent()
        cpu_util = psutil._cpu_busy_time(cpu_times) #pylint: disable=protected-access
        virtual_memory = psutil.virtual_memory()
        mem_percent_util = virtual_memory.percent
        new_resource_util_obj = ResourceUtil()
        new_resource_util_obj.interval = period
        new_resource_util_obj.cpu_utilization = cpu_util
        new_resource_util_obj.cpu_user = cpu_times.user
        new_resource_util_obj.cpu_nice = cpu_times.nice
        new_resource_util_obj.cpu_system = cpu_times.system
        new_resource_util_obj.cpu_idle = cpu_times.idle
        new_resource_util_obj.cpu_iowait = cpu_times.iowait
        new_resource_util_obj.cpu_irq = cpu_times.irq
        new_resource_util_obj.cpu_softirq = cpu_times.softirq
        new_resource_util_obj.cpu_steal = cpu_times.steal #pylint: disable=no-member
        new_resource_util_obj.cpu_guest = cpu_times.guest #pylint: disable=no-member
        new_resource_util_obj.cpu_guest_nice = cpu_times.guest_nice #pylint: disable=no-member
        new_resource_util_obj.mem_utilization = mem_percent_util
        new_resource_util_obj.mem_total = virtual_memory.total
        new_resource_util_obj.mem_used = virtual_memory.used

        disk_mode = cmd_output.split(',')[0]
        if disk_mode == 'ro':
            new_resource_util_obj.is_disk_read_only = True
        else:
            new_resource_util_obj.is_disk_read_only = False

        disk_partitions = psutil.disk_partitions(all=False)
        disk_utilization_json = {}

        '''
        create a yaml at DISK_FSTYPE_IGNORE_LIST path like below
        fstype:
          - tmpfs
        '''
        if os.path.exists(DefaultPaths.DISK_FSTYPE_IGNORE_LIST):
            try:
                if not len(self._DISK_IGNORE_CACHE) == 0:
                    with open(DefaultPaths.DISK_FSTYPE_IGNORE_LIST, 'r') as f:
                        self._DISK_IGNORE_CACHE = yaml.safe_load(f)
            except Exception:  # pylint: disable=broad-except
                pass
        ignore_list = ['squashfs']
        if 'fstype' in self._DISK_IGNORE_CACHE.keys():
            ignore_list.extend([fst for fst in self._DISK_IGNORE_CACHE['fstype']])
        for sdiskpart in disk_partitions:
            if sdiskpart.fstype in ignore_list:
                continue
            if sdiskpart.mountpoint == '/':
                new_resource_util_obj.root_disk = sdiskpart.device
            disk_path = sdiskpart.device
            disk_usage = psutil.disk_usage(sdiskpart.mountpoint)
            disk_utilization_json[disk_path] = {
                'percent': disk_usage.percent,
                'total': disk_usage.total,
                'used': disk_usage.used
            }

        new_resource_util_obj.disk_utilization = disk_utilization_json

        new_resource_util_obj.save()

        resource_state_arr = []
        resource_state_arr.append(new_resource_util_obj)
        if fullstate is True:
            save_fullstate(resource_state_arr,
                           ResourceUtil().__class__.__name__,
                           should_send_gnmi=True)

        return ServiceStatus.OK
