#
# Copyright (c) 2020-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#

import time
from netq_lib.orm import HOSTNAME
from netq_lib.orm.redisdb.models import RunningConfigMon
from netq_lib.common.enums import ServiceStatus
from netq_lib.common.utils import save_fullstate
from netq_agent.parsers.common.render import BaseRender

class RunningConfigMonRender(BaseRender):
    def __init__(self):
        super(RunningConfigMonRender, self).__init__()
        self.model = RunningConfigMon

    def get_config_dict(self):
        raise NotImplementedError

    def build_config_diff_objs(self, command_name, command_result, current_modify_time):
        new_config_mon_obj = RunningConfigMon()
        new_config_mon_obj.commandname = command_name
        new_config_mon_obj.last_modify_time = current_modify_time
        new_config_mon_obj.result = command_result.op.decode()
        new_config_mon_obj.save()
        return new_config_mon_obj

    def parse(self, cmd_output, svc_failed=False, **kwargs):
        try:
            conf_dict = self.get_config_dict()
        except ValueError:
            m_configMon = RunningConfigMon.memmodel()
            for entry in m_configMon.query.filter(hostname=HOSTNAME):
                old_config_mon_ent = RunningConfigMon()
                old_config_mon_ent.copy(entry)
                old_config_mon_ent.delete()
            return ServiceStatus.FAIL

        fullstate = kwargs.get('fullstate')

        # grab current list of configMon objects
        m_configMon = RunningConfigMon.memmodel()
        old_configmon_objs = []
        full_state_arr = []
        deleteCommandist = []
        if m_configMon:
            old_configmon_objs = list(m_configMon.query.filter(hostname=HOSTNAME))
        if conf_dict.get('running-config-diff'):
            for commandname, command in list(conf_dict.get('running-config-diff').items()):
                result = self.run_command(command)
                if result.success:
                    mem_command_result = ""
                    for old_entry in old_configmon_objs:
                        if old_entry.commandname == commandname:
                            mem_command_result = old_entry.result
                            break
                    current_modify_time = time.time()

                    if fullstate:
                        full_state_arr.append(self.build_config_diff_objs(commandname, result, current_modify_time))
                    elif result.op != mem_command_result:
                        self.build_config_diff_objs(commandname, result, current_modify_time)

                else:
                    deleteCommandist.append(commandname)
        for to_be_deleted_command in deleteCommandist:
            for old_obj in old_configmon_objs:
                if old_obj.commandname == to_be_deleted_command:
                    old_configmon_ent = RunningConfigMon()
                    old_configmon_ent.copy(old_obj)
                    old_configmon_ent.delete()
                    break


        if fullstate:
            save_fullstate(full_state_arr, RunningConfigMon.__name__)

        return ServiceStatus.OK
