#
# Copyright (c) 2020-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#

from netq_lib.orm.redisdb.models import Power
from netq_lib.common.enums import ServiceStatus
from netq_lib.common.utils import save_fullstate
from netq_lib.common.enums import Platform
from netq_agent.parsers.common.render import BaseRender

local_sensor_dict = dict()
FAIL_TEMP = -273
FAIL_FAN = -1

class SensorRender(BaseRender):
    def __init__(self): #pylint: disable=useless-super-delegation
        super(SensorRender, self).__init__()


class SensorPowerRender(BaseRender):
    def __init__(self):
        super(SensorPowerRender, self).__init__()
        self.model = Power

    def parse(self, cmd_output, svc_failed=False, **kwargs):
        fullstate = kwargs.get('fullstate')
        platform = kwargs.get('platform', '')
        random_platform = kwargs.get('random', '')
        power_arr = []
        required_fields = ['input', 'min', 'max']
        if Platform.VX in platform:
            self.handle_vx_platform(random_platform, power_arr, fullstate)

        if Platform.MELLANOX in platform:
            psu_output_list = [x.strip() for x in cmd_output.split('\n')]
            index = 0
            psu_output_list.append('')
            while index < len(psu_output_list):
                start = index
                found_psu = False
                while psu_output_list[index] != '':
                    if self.psu_found_check(psu_output_list[index]):
                        found_psu = True
                        break
                    index += 1
                index += 1
                if found_psu:
                    index = start
                else:
                    continue
                try:
                    if found_psu:
                        power_obj = Power()
                        power_obj.s_adapter_name = psu_output_list[index]
                        index += 1
                        while psu_output_list[index] != '':
                            voltage_prefix = None
                            if self.voltage_in_check(psu_output_list[index]):
                                voltage_prefix = psu_output_list[index + 1].split('_')[0]
                                voltage_attr = 's_voltage_in_'
                                index += 1
                            elif self.voltage_out_check(psu_output_list[index]):
                                voltage_prefix = psu_output_list[index + 1].split('_')[0]
                                voltage_attr = 's_voltage_out_'
                                index += 1
                            if voltage_prefix:
                                while psu_output_list[index].startswith(voltage_prefix):
                                    suffix = psu_output_list[index].split(':')
                                    data = float(suffix[1].strip())
                                    attr = suffix[0].split("_")[1]
                                    if attr in required_fields:
                                        setattr(power_obj, voltage_attr + attr, data)
                                    index += 1
                            power_prefix = None
                            if self.power_in_check(psu_output_list[index]):
                                power_prefix = psu_output_list[index + 1].split('_')[0]
                                power_attr = 's_power_in_'
                                index += 1
                            elif self.power_out_check(psu_output_list[index]):
                                power_prefix = psu_output_list[index + 1].split('_')[0]
                                power_attr = 's_power_out_'
                                index += 1
                            if power_prefix:
                                while psu_output_list[index].startswith(power_prefix):
                                    suffix = psu_output_list[index].split(':')
                                    data = float(suffix[1].strip())
                                    attr = suffix[0].split("_")[1]
                                    if attr in required_fields:
                                        setattr(power_obj, power_attr + attr, data)
                                    index += 1
                            if not power_prefix and not voltage_prefix:
                                index += 1
                        if not fullstate:
                            power_obj.save()
                        power_arr.append(power_obj)
                except ValueError:
                    power_arr = []
                    break
        if fullstate:
            save_fullstate(power_arr, "Power", False)
        return ServiceStatus.OK
