#
# Copyright (c) 2017-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#


from netq_lib.orm.redisdb.models import Lldp
from netq_agent.parsers.common.lldpd import LldpRender


class CumulusLldpRender(LldpRender):
    def __init__(self): #pylint: disable=useless-super-delegation
        super(CumulusLldpRender, self).__init__()

    def parse_lldp(self, lldp, platform):
        if platform and 'ubuntu' in platform.lower():
            new_lldp_ent = self.parse_host_lldp(lldp)
        elif platform and 'centos' in platform.lower():
            new_lldp_ent = self.parse_host_lldp(lldp)
        elif platform and 'red hat' in platform.lower():
            new_lldp_ent = self.parse_host_lldp(lldp)
        elif platform and 'rhel' in platform.lower():
            new_lldp_ent = self.parse_host_lldp(lldp)
        else:
            new_lldp_ent = self.parse_cl_lldp(lldp)
        return new_lldp_ent

    def parse_cl_lldp(self, lldp):
        lldpobj = Lldp()

        lldpobj.ifname = str(lldp['name'])
        try:
            lldpobj.peer_hostname = str(
                lldp['chassis'][0]['name'][0]['value']
            )
        except Exception:
            # chassis id will always be present
            lldpobj.peer_hostname = (
                str(lldp['chassis'][0]['id'][0]['type']) + ":" +
                str(lldp['chassis'][0]['id'][0]['value'])
            )

        if '.' in lldpobj.peer_hostname:
            lldpobj.peer_hostname = (
                lldpobj.peer_hostname.split('.')[0]
            )
        if lldp['port'][0]['id'][0]['type'] == 'ifname':
            lldpobj.peer_ifname = str(lldp['port'][0]['id'][0]['value'])
        else:
            # port id will be present always - sometimes as mac sub-type
            lldpobj.peer_ifname = (str(lldp['port'][0]['id'][0]['type']) + ":" +
                                   str(lldp['port'][0]['id'][0]['value']))

        try:
            descr = lldp['chassis'][0]['descr'][0]['value']
        except Exception:
            descr = 'unknown'

        words = descr.split()
        if words[0] == 'Cumulus':
            if len(words) > 3:
                lldpobj.lldp_peer_os = str(
                    ' '.join(words[:2]))
                if words[2] == 'version':
                    try:
                        if "(" in words[3]:
                            lldpobj.lldp_peer_osv = (
                                words[4].split(')')[1])
                        else:
                            lldpobj.lldp_peer_osv = (
                                words[3])
                    except Exception:
                        lldpobj.lldp_peer_osv = (
                            ' '.join(words[3:4]))
                else:
                    lldpobj.lldp_peer_osv = 'unknown'
            else:
                lldpobj.lldp_peer_os = words[0]
                lldpobj.lldp_peer_osv = 'unknown'
        elif words[0] == 'Ubuntu':
            lldpobj.lldp_peer_os = words[0]
            if len(words) > 1:
                if "16" in words[1]:
                    lldpobj.lldp_peer_osv = words[1]
                else:
                    lldpobj.lldp_peer_osv = 'unknown'
            else:
                lldpobj.lldp_peer_osv = 'unknown'
        elif words[0] == 'SONiC':
            lldpobj.lldp_peer_os = words[0]
            peer_info = dict()
            try:
                for info in descr.split(' - '):
                    [k, v] = info.split(':')
                    peer_info[k.strip()] = v.strip()
            except ValueError:
                pass    # continue, nothing to be done.
            lldpobj.lldp_peer_osv = peer_info.get('SONiC Software Version')
        else:
            lldpobj.lldp_peer_os = words[0]
            lldpobj.lldp_peer_osv = 'unknown'

        try:
            for cap in lldp['chassis'][0]['capability']:
                if cap['type'] == 'Bridge':
                    lldpobj.lldp_peer_bridge = cap['enabled']
                elif cap['type'] == 'Router':
                    lldpobj.lldp_peer_router = cap['enabled']
                elif cap['type'] == 'Station':
                    lldpobj.lldp_peer_station = cap['enabled']
        except Exception:
            pass

        if lldpobj.peer_hostname != 'cumulus':
            lldpobj.save()
        return lldpobj


def process_lldpctl(cmd_output, svc_failed=False, **kwargs):
    parser = CumulusLldpRender()
    return parser.parse(cmd_output, svc_failed, **kwargs)
