#
# Copyright (c) 2017-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#

import subprocess
from netq_lib.common.utils import subprocess_wrapper
from netq_agent.parsers.common.ports import PortsRender


class CumulusPortsRender(PortsRender):
    def __init__(self): #pylint: disable=useless-super-delegation
        super(CumulusPortsRender, self).__init__()

    @staticmethod
    def _get_ethtool_path():
        return '/sbin/ethtool'

    def ethtool_fetch_test_serial_number(self, port_name):
        test_serial_number = None
        if self.is_cl_platform:
            try:
                test_serial_number = subprocess_wrapper(
                    ['/sbin/ethtool', '-m', port_name, 'offset', '168', 'length',
                     '16', 'raw', 'on'])
            except subprocess.CalledProcessError:
                test_serial_number = None
            except UnicodeDecodeError:
                test_serial_number = None
        return test_serial_number

    @staticmethod
    def _is_optical_port(ethtool_m_dict):
        if 'Optical diagnostics support' in ethtool_m_dict and 'Yes' in ethtool_m_dict['Optical diagnostics support']:
            return True
        elif 'Transmitter technology' in ethtool_m_dict and 'Copper' not in ethtool_m_dict['Transmitter technology']:
            return True

        return False


def process_ports(cmd_output, svc_failed=False, **kwargs):
    ports_render.parse(cmd_output, svc_failed, **kwargs)


ports_render = CumulusPortsRender()
