#
# Copyright (c) 2019-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#



import ujson as json

from netq_lib.orm import HOSTNAME
from netq_lib.orm.redisdb.models import Ptm
from netq_lib.common.enums import ServiceStatus
from netq_lib.common.utils import save_fullstate
from netq_agent.parsers.common.render import BaseRender


class CumulusPtmRender(BaseRender):
    def __init__(self):
        super(CumulusPtmRender, self).__init__()
        self.model = Ptm

    def build_ptm_objs_full_state(self, cblStatus, port, expNbr, actNbr, full_state_arr):

        new_config_mon_obj = Ptm()
        new_config_mon_obj.cbl_status = cblStatus
        new_config_mon_obj.port = port
        new_config_mon_obj.exp_nbr = expNbr
        new_config_mon_obj.act_nbr = actNbr
        full_state_arr.append(new_config_mon_obj)

    def build_ptm_objs(self, values, new_ptm_objs, full_state_arr, fullstate):
        if fullstate:
            self.build_ptm_objs_full_state(str(values['cbl status']), str(values['port']), str(values['exp nbr']), str(values['act nbr']), full_state_arr)
        else:
            new_ptm_obj = Ptm()
            new_ptm_obj.cbl_status = str(values['cbl status'])
            new_ptm_obj.port = str(values['port'])
            new_ptm_obj.exp_nbr = str(values['exp nbr'])
            new_ptm_obj.act_nbr = str(values['act nbr'])
            new_ptm_obj.save()
            new_ptm_objs.append(new_ptm_obj)

    def checkIfTopologyFileExists(self, json_out):
        flag = True
        for values in list(json_out.values()):
            if "cmd_status" in values:
                if values["cmd_status"] == "fail":
                    flag = False
        return flag

    def parse(self, cmd_output, svc_failed=False, **kwargs):
        """Build ptm info
        """
        fullstate = kwargs.get('fullstate')

        try:
            # sometimes we get embedded double-quotes in json strings
            # that are added with backslash. these get removed
            # when using 'string-escape'. so we need to
            # double 'escape' the strings
            # we need to use 'string-escape' to handle Cisco
            # devices that split descriptions into multi-line outputs
            cmd_output = cmd_output.replace("\\\"", "\\\\\"")

            if "ptmctl: command not found" in str(cmd_output) \
                    or "Unable to connect to PTMD [Connection refused]" \
                    in str(cmd_output):
                return ServiceStatus.FAIL
            json_out = json.loads(cmd_output) #pylint: disable=no-member
            if not self.checkIfTopologyFileExists(json_out):
                return ServiceStatus.FAIL
        except ValueError:
            m_Ptm = Ptm.memmodel()
            for entry in m_Ptm.query.filter(hostname=HOSTNAME):
                old_ptm_ent = Ptm()
                old_ptm_ent.copy(entry)
                old_ptm_ent.delete()
            dummy = []
            if fullstate:
                save_fullstate(dummy, Ptm().__class__.__name__)
            return ServiceStatus.FAIL


        # grab current list of Ptm objects
        new_ptm_objs = []
        full_state_arr = []

        for values in list(json_out.values()):
            self.build_ptm_objs(values, new_ptm_objs, full_state_arr, fullstate)

        if fullstate:
            save_fullstate(full_state_arr, Ptm.__name__)

        return ServiceStatus.OK


def process_ptmctl(cmd_output, svc_failed=False, **kwargs):
    render = CumulusPtmRender()
    return render.parse(cmd_output, svc_failed, **kwargs)
