#
# Copyright (c) 2019-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#

import ujson as json
from netq_lib.orm.redisdb.models import ForwardingResource
from netq_lib.orm.redisdb.models import ACLResource
from netq_lib.common.enums import ServiceStatus
from netq_lib.common.utils import save_fullstate
from netq_agent.parsers.common.tcam_resource import TcamRender


class CumulusTcamRender(TcamRender):
    def __init__(self):
        super(CumulusTcamRender, self).__init__()
        self.model = ACLResource

    def parse(self, cmd_output, svc_failed=False, **kwargs):
        self._logger = kwargs.get('logger')
        fullstate = kwargs.get('fullstate')
        try:
            json_out = json.loads(cmd_output) #pylint: disable=no-member
        except ValueError:
            return ServiceStatus.FAIL

        forwarding_resource = ForwardingResource()
        acl_resource = ACLResource()
        forwarding_resource_attr = dir(forwarding_resource)
        acl_resource_attr = dir(acl_resource)

        for key, util in list(json_out.items()):
            if 'max' not in util or 'count' not in util:
                continue
            util_json = {'max': util['max'], 'usage': util['count']}
            if key in forwarding_resource_attr:
                setattr(forwarding_resource, key, util_json)

            if key in acl_resource_attr:
                setattr(acl_resource, key, util_json)

        forwarding_resource.save()
        acl_resource.save()

        if fullstate is True:
            acl_resource_arr = []
            acl_resource_arr.append(acl_resource)
            save_fullstate(acl_resource_arr, ACLResource().__class__.__name__)

            fwd_res_arr = []
            fwd_res_arr.append(forwarding_resource)
            save_fullstate(fwd_res_arr, ForwardingResource().__class__.__name__)

        return ServiceStatus.OK


def process_tcam_resource(cmd_output, svc_failed=False, **kwargs):
    render = CumulusTcamRender()
    return render.parse(cmd_output, svc_failed, **kwargs)
