#
# Copyright (c) 2020-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#

from netq_lib.orm.redisdb.models import ConfigMon
from netq_agent.parsers.common.configMon import ConfigMonRender


class SonicConfigMonRender(ConfigMonRender):

    def __init__(self):
        super(SonicConfigMonRender, self).__init__()
        self._container_names = {
            # Filename : Container Name
            'frr': 'bgp',
            'lldpd': 'lldp'
        }

    def get_config_dict(self):
        conf_dict = {
            'config-diff': {
                'config_db': '/etc/sonic/config_db.json',
                'frr': '/etc/frr/bgpd.conf,/etc/frr/zebra.conf',
                'lldpd': '/etc/lldpd.conf',
                'ntp': '/etc/ntp.conf'
            }
        }
        return conf_dict

    def has_special_handling(self, filename):
        if filename in ['lldpd', 'frr']:
            return True

        return False

    def get_last_modify_time(self, filename, filepath):
        timestamp = None
        if filename in ['lldpd', 'frr']:
            # the conf file is present within the docker instance
            container_name = self._container_names.get(filename)
            if not container_name:
                self._logger.warn('Failed to fetch associated container name for {}'.format(filename))
                return timestamp
            for path in filepath.split(','):
                result = self.run_command('docker exec {} date -r {} +%s'.format(container_name, path))
                if result.success and result.op:
                    if not timestamp:
                        timestamp = float(result.op.decode().strip())
                    else:
                        if timestamp < float(result.op.decode().strip()):
                            timestamp = float(result.op.decode().strip())

        if not timestamp:
            self._logger.warn("Failed to fetch current time for {}".format(filename))

        return timestamp

    @staticmethod
    def _get_file_header(filename, path):
        return "!\n" \
               "! {}\n" \
               "!\n".format(path)

    def build_platform_specific_objs(self, filename, filepath, fullstate, mem_modify_time, current_modify_time):
        new_config_mon_obj = None
        if filename in ['lldpd', 'frr']:
            # the conf file is present within the docker instance
            container_name = self._container_names.get(filename)
            if not container_name:
                self._logger.warn('Failed to fetch associated container name for {}'.format(filename))
                return None
            for path in filepath.split(','):
                result = self.run_command('docker exec {} cat {}'.format(container_name, path))
                if result.success and result.op:
                    if int(current_modify_time) > mem_modify_time or fullstate:
                        if not new_config_mon_obj:
                            new_config_mon_obj = ConfigMon()
                            new_config_mon_obj.file_name = filename
                            new_config_mon_obj.last_modify_time = current_modify_time
                            new_config_mon_obj.file_content = result.op.decode()
                        else:
                            new_config_mon_obj.file_content = new_config_mon_obj.file_content + result.op.decode()
                else:
                    self._logger.info("Failed to execute command - docker exec {} cat {}".format(container_name, path))
        return new_config_mon_obj


def process_config_mon(cmd_output, svc_failed=False, **kwargs):
    render = SonicConfigMonRender()
    return render.parse(cmd_output, svc_failed, **kwargs)
