#
# Copyright (c) 2019-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#

from netq_agent.parsers.common.dpkg import DpkgRender
os_version = "N/A"

class SonicDpkgRender(DpkgRender):
    def __init__(self):
        super(SonicDpkgRender, self).__init__()

    def get_os_version(self):
        global os_version
        if os_version == 'N/A':
            result = self.run_command("show version")
            if result.success and result.op:
                try:
                    for line in result.op.decode().split("\n"):
                        if 'SONiC Software Version:' in line:
                            os_version = line.split(':')[1].strip()
                except (IndexError, ValueError) as ex:
                    self._logger.error("Failed to process os info: {}".format(ex))
                    raise ex
            else:
                self._logger.info("DPKG: failed to run show version command")
        return os_version


def process_dpkg(cmd_output, svc_failed=False, **kwargs):
    parser = SonicDpkgRender()
    return parser.parse(cmd_output, svc_failed, **kwargs)
