#
# Copyright (c) 2020-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#

import re
from netq_agent.parsers.common.render import BaseRender


def process_feature_status(logger, feature_name):
    render = BaseRender()
    # Feature             State     AutoRestart    SetOwner
    # ------------------  --------  -------------  ----------
    cmd = 'show feature status {}'.format(feature_name)
    header = "Feature             State     AutoRestart    SetOwner".split()
    result = render.run_command(cmd)
    if result.success and result.op:
        idx = 0
        matches_positions = []
        table_margin = 0
        for row in result.op.decode().split("\n"):
            if idx == 0:
                if header != row.split():
                    logger.error("Command 'show feature status' header validation failed. Details {}".format(row))
                    return False
            elif idx == 1:
                matches = re.finditer('  ', row)
                matches_positions = [match.start() for match in matches]
                matches_positions.append(None)
                matches_positions.insert(0,0)
                table_margin = len(row)
            elif len(row) == 0:
                idx += 1
                continue
            else:
                data = [row[matches_positions[i]:matches_positions[i+1]].lstrip() for i in range(len(matches_positions)-1)]
                name = data[0]
                state = data[1]
                if feature_name != name:
                    logger.error("Invalid output received for show feature status {}".format(feature_name))
                    return False
                return True if state == 'enabled' else False
            idx += 1
    return False
