#
# Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#


import re
import json
import subprocess

from netq_lib.common.utils import subprocess_wrapper
from netq_agent.parsers.common.render import BaseRender


class SonicIfAliasRender(BaseRender):
    def __init__(self, logger):
        super(SonicIfAliasRender, self).__init__()
        self._logger = logger

    def parse(self):
        status_cmd_result = dict()
        try:
            show_cmd_dump = subprocess_wrapper('show interface status'.split())
        except subprocess.CalledProcessError:
            return status_cmd_result

        if show_cmd_dump:
            #     Interface    Lanes    Speed    MTU    FEC       Alias           Vlan    Oper    Admin            Type    Asym PFC
            # -------------  -------  -------  -----  -----  ----------  -------------  ------  -------  --------------  ----------
            header = "    Interface    Lanes    Speed    MTU    FEC       Alias           Vlan    Oper    Admin            Type    Asym PFC".split()
            idx = 0
            matches_positions = []
            table_margin = 0
            for row in show_cmd_dump.split("\n"):
                if idx == 0:
                    if header != row.split():
                        self._logger.error("Command 'show interface status' header validation failed")
                        break
                elif idx == 1:
                    matches = re.finditer('  ', row)
                    matches_positions = [match.start() for match in matches]
                    matches_positions.append(None)
                    matches_positions.insert(0,0)
                    table_margin = len(row)
                elif len(row) != table_margin:
                    self._logger.warn("Invalid row {}".format(row))
                else:
                    data = [row[matches_positions[i]:matches_positions[i+1]].lstrip() for i in range(len(matches_positions)-1)]
                    status_cmd_result[data[0]] = {
                        'ifalias': data[5],
                        'oper_state': data[7],
                        'admin_state': data[8]
                    }
                idx = idx + 1
        return status_cmd_result


def get_ifalias_mapping(logger):
    render = SonicIfAliasRender(logger)
    result = render.parse()
    return result


def update_ifalias_information(json_out, logger):
    result = get_ifalias_mapping(logger)
    data = json.loads(json_out)
    for link in data:
        if link.get('ifname') in result:
            link['ifalias'] = result.get(link.get('ifname')).get('ifalias')
            link['oper_state'] = result.get(link.get('ifname')).get('oper_state')
            if link['oper_state'] == 'up':
                link['down_reason'] = ''
            link['admin_state'] = result.get(link.get('ifname')).get('admin_state')
    return json.dumps(data)

