#
# Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#

from netq_agent.parsers.sonic.redis_db.client import SonicDBConnector, SonicDB, SonicTable, SonicDBRender, sonic_data_get
from netq_lib.orm.redisdb.models import ProcNetDevInterface, EthtoolInterface


class EthtoolHandler(SonicDBRender):
    MODEL = EthtoolInterface

    def __init__(self):
        super(EthtoolHandler, self).__init__()

    def db_sources(self):
        return {
            (SonicDB.COUNTERS_DB, SonicTable.COUNTERS_DB_COUNTERS_PORT_NAME_MAP): self.counters_db_counters_port_name_map,
            (SonicDB.COUNTERS_DB, SonicTable.COUNTERS_DB_COUNTERS): self.counters_db_counters
        }

    def counters_db_counters_port_name_map(self):
        pass

    def counters_db_counters(self):
        pass

    def _create_proc(self, ifname, rx_bytes=-1, rx_drop=-1, rx_errs=-1, rx_frame=-1, rx_multicast=-1, rx_packets=-1, tx_bytes=-1,
                    tx_carrier=-1, tx_colls=-1, tx_drop=-1, tx_errs=-1, tx_packets=-1, rx_broadcast=-1, tx_broadcast=-1, tx_multicast=-1):
        # fifo and compressed related counters are intentionally left out
        iface = ProcNetDevInterface()
        iface.ifname = ifname
        iface.rx_bytes = rx_bytes
        iface.rx_drop = rx_drop
        iface.rx_errs = rx_errs
        iface.rx_frame = rx_frame
        iface.rx_multicast = rx_multicast
        iface.rx_packets = rx_packets
        iface.tx_bytes = tx_bytes
        iface.tx_carrier = tx_carrier
        iface.tx_colls = tx_colls
        iface.tx_drop = tx_drop
        iface.tx_errs = tx_errs
        iface.tx_packets = tx_packets
        iface.rx_broadcast = rx_broadcast
        iface.tx_broadcast = tx_broadcast
        iface.tx_multicast = tx_multicast
        return iface

    def _attr_db_key_mapping(self):
        return {
            'hw_if_in_octets': 'SAI_PORT_STAT_IF_IN_OCTETS',
            'hw_if_in_ucast_pkts': 'SAI_PORT_STAT_IF_IN_UCAST_PKTS',
            'hw_if_in_bcast_pkts': 'SAI_PORT_STAT_IF_IN_BROADCAST_PKTS',
            'hw_if_in_mcast_pkts': 'SAI_PORT_STAT_IF_IN_MULTICAST_PKTS',
            'hw_if_out_octets': 'SAI_PORT_STAT_IF_OUT_OCTETS',
            'hw_if_out_ucast_pkts': 'SAI_PORT_STAT_IF_OUT_UCAST_PKTS',
            'hw_if_out_mcast_pkts': 'SAI_PORT_STAT_IF_OUT_MULTICAST_PKTS',
            'hw_if_out_bcast_pkts': 'SAI_PORT_STAT_IF_OUT_BROADCAST_PKTS',
            'hw_if_in_discards': 'SAI_PORT_STAT_IF_IN_DISCARDS',
            #'hw_if_in_l3_drops': '',
            #'hw_if_in_buffer_drops': '',
            #'hw_if_in_acl_drops': '',
            #'hw_if_in_dot3_length_errors': '',
            'hw_if_in_errors': 'SAI_PORT_STAT_IF_IN_ERRORS',
            #'soft_in_errors': '',
            #'soft_in_drops': '',
            #'soft_in_frame_errors': '',
            'hw_if_out_discards': 'SAI_PORT_STAT_IF_OUT_DISCARDS',
            'hw_if_out_errors': 'SAI_PORT_STAT_IF_OUT_ERRORS',
            #'hw_if_out_q_drops': '',
            #'hw_if_out_non_q_drops': '',
            #'soft_out_errors': '',
            #'soft_out_drops': '',
            #'soft_out_tx_fifo_full': '',
            #'hw_if_out_q_len': '',
            #'hw_if_in_dot3_frame_errors': '',
            #'hw_if_symbol_errors': '',
            #'hw_if_alignment_errors': '',
            'hw_if_undersize_errors': 'SAI_PORT_STAT_ETHER_STATS_UNDERSIZE_PKTS',
            'hw_if_oversize_errors': 'SAI_PORT_STAT_ETHER_TX_OVERSIZE_PKTS',
            'hw_if_jabber_errors': 'SAI_PORT_STAT_ETHER_STATS_JABBERS',
            #'hw_if_in_pause_pkt': '',
            #'hw_if_out_pause_pkt': '',
            'hw_if_in_pfc0_pkt': 'SAI_PORT_STAT_PFC_0_RX_PKTS',
            'hw_if_out_pfc0_pkt': 'SAI_PORT_STAT_PFC_0_TX_PKTS',
            'hw_if_in_pfc1_pkt': 'SAI_PORT_STAT_PFC_1_RX_PKTS',
            'hw_if_out_pfc1_pkt': 'SAI_PORT_STAT_PFC_1_TX_PKTS',
            'hw_if_in_pfc2_pkt': 'SAI_PORT_STAT_PFC_2_RX_PKTS',
            'hw_if_out_pfc2_pkt': 'SAI_PORT_STAT_PFC_2_TX_PKTS',
            'hw_if_in_pfc3_pkt': 'SAI_PORT_STAT_PFC_3_RX_PKTS',
            'hw_if_out_pfc3_pkt': 'SAI_PORT_STAT_PFC_3_TX_PKTS',
            'hw_if_in_pfc4_pkt': 'SAI_PORT_STAT_PFC_4_RX_PKTS',
            'hw_if_out_pfc4_pkt': 'SAI_PORT_STAT_PFC_4_TX_PKTS',
            'hw_if_in_pfc5_pkt': 'SAI_PORT_STAT_PFC_5_RX_PKTS',
            'hw_if_out_pfc5_pkt': 'SAI_PORT_STAT_PFC_5_TX_PKTS',
            'hw_if_in_pfc6_pkt': 'SAI_PORT_STAT_PFC_6_RX_PKTS',
            'hw_if_out_pfc6_pkt': 'SAI_PORT_STAT_PFC_6_TX_PKTS',
            'hw_if_in_pfc7_pkt': 'SAI_PORT_STAT_PFC_7_RX_PKTS',
            'hw_if_out_pfc7_pkt': 'SAI_PORT_STAT_PFC_7_TX_PKTS',
            #'hw_if_out_wred_drops': '',
            #'hw_if_out_q0_wred_drops': '',
            #'hw_if_out_q1_wred_drops': '',
            #'hw_if_out_q2_wred_drops': '',
            #'hw_if_out_q3_wred_drops': '',
            #'hw_if_out_q4_wred_drops': '',
            #'hw_if_out_q5_wred_drops': '',
            #'hw_if_out_q6_wred_drops': '',
            #'hw_if_out_q7_wred_drops': '',
            #'hw_if_out_q8_wred_drops': '',
            #'hw_if_out_q9_wred_drops': ''
        }   
            
    def _create_ethtool_stat_obj(self, ifname, data):
        ethtool_obj = EthtoolInterface()
        ethtool_obj.ifname = ifname
        if data:
            attr_map = self._attr_db_key_mapping()
            for attr_name, db_key in attr_map.items():
                try:
                    setattr(ethtool_obj, attr_name, int(data.get(db_key, -1)))
                except (ValueError, KeyError) as ex:
                    self._logger.debug('Failed to set attr {} for ethtool object'.format(attr_name))
                    pass
        return ethtool_obj

    def process_fullstate(self, data):
        port_map = sonic_data_get(data,SonicDB.COUNTERS_DB, SonicTable.COUNTERS_DB_COUNTERS_PORT_NAME_MAP)
        if not port_map:
            self._logger.info('Failed to retrieve COUNTERS_PORT_NAME_MAP')
            return
        for ifname, oid in port_map.items():
            record = sonic_data_get(data,SonicDB.COUNTERS_DB, SonicTable.COUNTERS_DB_COUNTERS).get(oid)
            if record:
                # Ethtool Stats
                obj = self._create_ethtool_stat_obj(ifname, record)
                if obj:
                    obj.save()
                # Interface Stats
                proc_stat_record = self._create_proc(ifname,
                                                     rx_bytes=int(record.get('SAI_PORT_STAT_IF_IN_OCTETS', -1)),
                                                     rx_drop=int(record.get('SAI_PORT_STAT_IF_IN_DISCARDS', -1)),
                                                     rx_errs=int(record.get('SAI_PORT_STAT_IF_IN_ERRORS', -1)),
                                                     rx_frame=int(record.get('SAI_PORT_STAT_IF_IN_UCAST_PKTS')) +
                                                        int(record.get('SAI_PORT_STAT_IF_IN_BROADCAST_PKTS')) +
                                                        int(record.get('SAI_PORT_STAT_IF_IN_MULTICAST_PKTS')) if (
                                                            record.get('SAI_PORT_STAT_IF_IN_UCAST_PKTS') and
                                                            record.get('SAI_PORT_STAT_IF_IN_BROADCAST_PKTS') and
                                                            record.get('SAI_PORT_STAT_IF_IN_MULTICAST_PKTS')) else -1,
                                                     rx_multicast=int(record.get('SAI_PORT_STAT_IF_IN_MULTICAST_PKTS', -1)),
                                                     rx_packets=int(record.get('SAI_PORT_STAT_IF_IN_UCAST_PKTS')) +
                                                        int(record.get('SAI_PORT_STAT_IF_IN_BROADCAST_PKTS')) +
                                                        int(record.get('SAI_PORT_STAT_IF_IN_MULTICAST_PKTS')) if (
                                                            record.get('SAI_PORT_STAT_IF_IN_UCAST_PKTS') and
                                                            record.get('SAI_PORT_STAT_IF_IN_BROADCAST_PKTS') and
                                                            record.get('SAI_PORT_STAT_IF_IN_MULTICAST_PKTS')) else -1,
                                                     tx_bytes=int(record.get('SAI_PORT_STAT_IF_OUT_OCTETS', -1)),
                                                     tx_drop=int(record.get('SAI_PORT_STAT_IF_OUT_DISCARDS')),
                                                     tx_errs=int(record.get('SAI_PORT_STAT_IF_OUT_ERRORS')),
                                                     tx_packets=int(record.get('SAI_PORT_STAT_IF_OUT_UCAST_PKTS')) +
                                                        int(record.get('SAI_PORT_STAT_IF_OUT_BROADCAST_PKTS')) +
                                                        int(record.get('SAI_PORT_STAT_IF_OUT_MULTICAST_PKTS')) if (
                                                            record.get('SAI_PORT_STAT_IF_OUT_UCAST_PKTS') and
                                                            record.get('SAI_PORT_STAT_IF_OUT_BROADCAST_PKTS') and
                                                            record.get('SAI_PORT_STAT_IF_OUT_MULTICAST_PKTS')) else -1,
                                                     rx_broadcast=int(record.get('SAI_PORT_STAT_IF_IN_BROADCAST_PKTS', -1)),
                                                     tx_broadcast=int(record.get('SAI_PORT_STAT_IF_OUT_BROADCAST_PKTS', -1)),
                                                     tx_multicast=int(record.get('SAI_PORT_STAT_IF_OUT_MULTICAST_PKTS', -1)))
                if proc_stat_record:
                    proc_stat_record.save()


def process_counters(*args, **kwargs):
    logger = kwargs.get('logger')
    render.process(logger)


render = EthtoolHandler()
