#
# Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#

import re
from netq_lib.orm.redisdb.models import ForwardingResource
from netq_lib.common.enums import ServiceStatus
from netq_lib.common.utils import save_fullstate
from netq_agent.parsers.common.tcam_resource import TcamRender


class SonicTcamRender(TcamRender):
    def __init__(self):
        super(SonicTcamRender, self).__init__()

    def parse(self, cmd_output, svc_failed=False, **kwargs):
        self._logger = kwargs.get('logger')
        fullstate = kwargs.get('fullstate')
        fwd_resource = None

        data = cmd_output.splitlines()
        if not data:
            self._logger.error("No data received for forwarding resources fetch command")
            return ServiceStatus.FAIL
        try:
            idx = data.index("Stage    Bind Point    Resource Name      Used Count    Available Count")
        except ValueError:
            self._logger.error("Unexpected data received for forwarding resources fetch command")
            return ServiceStatus.FAIL

        parser_input = '\n'.join(l.rstrip() for l in data[:idx] if l.strip())

        # Resource Name           Used Count    Available Count
        # --------------------  ------------  -----------------
        header = "Resource Name           Used Count    Available Count".split()
        idx = 0
        matches_positions = []
        table_margin = 0
        for row in parser_input.split("\n"):
            if idx == 0:
                if header != row.split():
                    self._logger.error("Command 'crm show resources all' header validation failed")
                    break
                fwd_resource = ForwardingResource()
            elif idx == 1:
                matches = re.finditer('  ', row)
                matches_positions = [match.start() for match in matches]
                matches_positions.append(None)
                matches_positions.insert(0,0)
                table_margin = len(row)
            elif len(row) != table_margin:
                self._logger.warn("Invalid row {}".format(row))
            else:
                data = [row[matches_positions[i]:matches_positions[i+1]].lstrip() for i in range(len(matches_positions)-1)]
                attr = data[0].strip()
                if attr == "ipv4_route":
                    util_json = {'max': str(int(data[1]) + int(data[2])), 'usage': data[1]}
                    fwd_resource.route_0_entry = util_json
                elif attr == "ipv6_route":
                    util_json = {'max': str(int(data[1]) + int(data[2])), 'usage': data[1]}
                    fwd_resource.route_1_entry = util_json
                elif attr == "fdb_entry":
                    util_json = {'max': str(int(data[1]) + int(data[2])), 'usage': data[1]}
                    fwd_resource.mac_entry = util_json
            idx = idx+1

        if fwd_resource:
            try:
                util_json = {'max': str(int(fwd_resource.route_0_entry['max']) + int(fwd_resource.route_1_entry['max'])),
                        'usage': str(int(fwd_resource.route_0_entry['usage']) + int(fwd_resource.route_1_entry['usage']))}
                fwd_resource.route_total_entry = util_json
            except KeyError:
                self._logger.debug("Failed to fetch route_total_entry")
                pass

            if not fullstate:
                fwd_resource.save()
            else:
                save_fullstate([fwd_resource], ForwardingResource().__class__.__name__)

        return ServiceStatus.OK


def process_tcam_resource(cmd_output, svc_failed=False, **kwargs):
    render = SonicTcamRender()
    return render.parse(cmd_output, svc_failed, **kwargs)

