#
# Copyright (c) 2019-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#

import sys
import time
from netq_lib.common import configuration

from multiprocessing import Process
from netq_agent.cmd.netq_grpc_agent import NetQAgentGrpcClient
from netq_lib.common.netq_pq import QueueElem

from netq_lib.common.enums import (
    Daemons,
    ConfKw,
)
from netq_lib.common.netq_pq import create_message_queues
from logging import StreamHandler
import logging

_PROC_NAME = 'netq-agent'

def get_logger(proc_name, conf=None):
    logger = logging.getLogger(proc_name)
    if len(logger.handlers):
        # Logger already configured
        return logger

    loglevel = 'debug'
    loglevel_dict = {'debug': logging.DEBUG,
                     'info': logging.INFO,
                     'warning': logging.WARNING,
                     'error': logging.ERROR}
    if conf:
        proc_conf = conf.get(proc_name, None)
        if proc_conf:
            loglevel = conf[proc_name].get('log_level', 'info')

    logging_level = loglevel_dict.get(loglevel, logging.INFO)

    handler = StreamHandler(sys.stdout)
    formatter = logging.Formatter(proc_name + '[%(process)d]: %(levelname)s: %(message)s')
    handler.setLevel(logging_level)
    handler.setFormatter(formatter)
    logger.setLevel(logging_level)
    logger.addHandler(handler)
    return logger

def send_data(log, pq):

    time.sleep(4)

    while True:
        log.debug("Sending Data")

        priority = 1

        pq_data = "{'param1':'value1', 'param2':'value2'}"

        q_elem = QueueElem(priority, "test_message", pq_data)

        try:
            pq.putq(q_elem)
        except Exception as e:
            print("Queue is full")
            time.sleep(1)

        time.sleep(.001)




def main():
    conf_dict = configuration.get_config(_PROC_NAME)

    log_obj = None
    pq = None

    stream_process = None
    if (conf_dict.get('netq-agent') and
            conf_dict.get('netq-agent').get(ConfKw.IS_OPTA_ENABLED, ConfKw.DEFAULT_IS_OPTA_ENABLED)):
        #Child Sync manager Process
        log_obj = get_logger(_PROC_NAME, conf_dict)
        pq, bp_pq = create_message_queues(conf=conf_dict, logger=log_obj)

        #Child GRPC Server Process
        grpc_server_inst = NetQAgentGrpcClient(conf_dict,log_obj, pq, bp_pq)
        stream_process = Process(target=grpc_server_inst.start_stream, args=(conf_dict,))
        stream_process.start()

    send_data(log_obj, pq)

    if stream_process:
        stream_process.join()

if __name__ == '__main__':
    main()


