#
# Copyright (c) 2018-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#

import eventlet
import eventlet.pools
import greenlet
import random


class TestThread(object):
    """Thread class.
    """

    def __init__(self):
        self.__function = None
        self.__pool = None
        self.__start_time = None
        self.__stop_checker = None
        self.green_thread = None
        self.pipeline = None

    def kill(self):
        eventlet.kill(self.green_thread)

    def link(self, stop_checker):
        self.__stop_checker = stop_checker
        self.green_thread.link(stop_checker)

    @classmethod
    def spawn(cls, epoch, pool, function, pipeline=None):
        thread = cls()
        thread.__function = function  # pylint: disable=protected-access
        thread.__pool = pool  # pylint: disable=protected-access
        thread.__start_time = epoch  # pylint: disable=protected-access
        thread.pipeline = pipeline
        if pipeline is not None:
            thread.green_thread = pool.spawn(function, pipeline)
        else:
            thread.green_thread = pool.spawn(function)
        return thread

    def stale(self, now):
        return now - self.__start_time >= Heartbeat.HEARTBEAT_DEATH

    def respawn(self, epoch):
        self.kill()
        thread = self.spawn(epoch, self.__pool, self.__function,
                            pipeline=self.pipeline)
        return thread
