#
# Copyright (c) 2018-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#

import grpc
import time
import json
from concurrent import futures

from netq_agent.interface.netq_pb2 import *
from netq_agent.interface.netq_pb2_grpc import *

from .lldp_msg import LLDP_STR

_ONE_DAY_IN_SECONDS = 60 * 60 * 24


class NetqServicer(NetqAgentStreamServiceServicer):

    def __init__(self):
        self.seq_no = 0
        pass

    def get_sequence_number(self):
        self.seq_no = self.seq_no + 1
        return self.seq_no

    def get_lldp_command_out(self):
        return LLDP_STR

    def message_builder(self):

        json_out = self.get_lldp_command_out()
        netq_message = NetqAgentMessage(aid="marvel-07",
                                        message_type=2,
                                        seqno=self.get_sequence_number(),
                                        ts=int(time.time()),
                                        type=2,
                                        message=json.dumps(json_out).encode('utf-8'))
        return netq_message

    def StreamMessage(self, request, context):

        print("Request received")

        msg = self.message_builder()

        yield msg


def serve():
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))

    add_NetqAgentStreamServiceServicer_to_server(NetqServicer(), server)
    server.add_insecure_port('[::]:8981')
    server.start()

    try:
        while True:
            time.sleep(_ONE_DAY_IN_SECONDS)

    except KeyboardInterrupt:
        server.stop(0)


if __name__ == '__main__':
    serve()
