#
# Copyright (c) 2019-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#

import requests
import json


class GetRequestError(requests.exceptions.RequestException):
    def __init__(self, url, headers, response=None):
        self.method = 'GET'
        self.url = url
        self.headers = headers
        self.response = response

    def __str__(self):
        if self.response:
            return '{} request to {} -H {} failed: {}'.format(self.method, self.url, str(self.headers),
                                                              json.dumps(self.response))
        else:
            return '{} request to {} -H {} failed'.format(self.method, self.url, str(self.headers))


class PostRequestError(requests.exceptions.RequestException):
    def __init__(self, url, headers, data, response=None):
        self.method = 'POST'
        self.url = url
        self.headers = headers
        self.data = data
        self.response = response

    def __str__(self):
        if self.response:
            return '{} request to {} -H {} -d {} failed: {}'.format(self.method, self.url,
                                                                    str(self.headers), self.data,
                                                                    json.dumps(self.response))
        else:
            return '{} request to {} -H {} -d {} failed'.format(self.method, self.url, str(self.headers), self.data)


class PutRequestError(requests.exceptions.RequestException):
    def __init__(self, url, headers, data, response=None):
        self.method = 'PUT'
        self.url = url
        self.headers = headers
        self.data = data
        self.response = response

    def __str__(self):
        if self.response:
            return '{} request to {} -H {} -d {} failed: {}'.format(self.method, self.url,
                                                                    str(self.headers), self.data,
                                                                    json.dumps(self.response))
        else:
            return '{} request to {} -H {} -d {} failed'.format(self.method, self.url, str(self.headers), self.data)


class BadStatusCode(requests.exceptions.RequestException):
    def __init__(self, response):
        self.response = response

    def __str__(self):
        if isinstance(self.response, dict) and self.response.get('message'):
            return self.response.get('message')
        else:
            return str(self.response)

